
#ifndef _WDATACRF_HPP_INCLUDED
#define _WDATACRF_HPP_INCLUDED

//------------------------------------------------------------------------
//
// WDataColumnReference
//
// This class represents data columns and is only of interest to
// implementors of new data access models (i.e. subclasses of WDataSource).
//
//------------------------------------------------------------------------

#ifndef _WOBJECT_HPP_INCLUDED
#include "wobject.hpp"
#endif
#ifndef _WEVENT_HPP_INCLUDED
#include "wevent.hpp"
#endif
#ifndef _WDATAOBJ_HPP_INCLUDED
#include "wdataobj.hpp"
#endif

class WCMCLASS WDataColumnReference : public WReferenceObject {

    private:

        WDataColumnReference( const WDataColumnReference & tr );
        WDataColumnReference & operator=( const WDataColumnReference & tr );

    protected:

        WDataColumnReference();

    public:

        ~WDataColumnReference();

        static WDataColumnReference *Allocate();

        virtual WBool   GetBound() const { return _bound; }
        virtual WBool   GetCaseSensitive() const { return _caseSensitive; }
        virtual WShort  GetIndex() const { return _index; }
        virtual WString GetLabel() const { return _label; }
        virtual WString GetName() const { return _name; }
        virtual WColumnDataType GetType() const { return _type; }
        virtual WNativeDataType GetNativeType() const
                  { return ColumnTypeToNativeType( _type, _unsigned ); }
        virtual WShort  GetScale() const { return _scale; }
        virtual WDWord  GetPrecision() const { return _precision; }
        virtual WDWord  GetLength() const { return _length; }
        virtual WShort  GetNullable() const { return _nullable; }
        virtual WDataSource *GetDataSource() const { return _source; }
        virtual WShort       GetUpdatable() const { return _updatable; }
        virtual WBool   GetUnsigned() const { return _unsigned; }
        virtual WDWord  GetDisplaySize() const { return _displaySize; }
        virtual WString GetTable() const { return _table; }
        virtual WString GetOwner() const { return _owner; }
        virtual WString GetQualifier() const { return _qualifier; }

        virtual WDataValue GetValue() const;
        virtual WBool      SetValue( const WDataValue & value );

        virtual WLong   QueryNumericAttribute( WUShort attr ) const;
        virtual WString QueryStringAttribute( WUShort attr ) const;

        WBool GetReadOnly() const { return GetUpdatable() == SQL_ATTR_READONLY; }

        static WNativeDataType ColumnTypeToNativeType( WColumnDataType ct,
                                                       WBool isUnsigned=TRUE );

    public:

        WDataSource     *_source;
        WInt             _index;
        WString          _label;
        WString          _name;
        WColumnDataType  _type;
        SWORD            _scale;
        UDWORD           _precision;
        SWORD            _nullable;
        SWORD            _updatable;
        WBool            _caseSensitive;
        UDWORD           _length;
        WBool            _unsigned;
        WDWord           _displaySize;
        WBool            _bound;
        WString          _table;
        WString          _owner;
        WString          _qualifier;
};

#endif
