/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %     Copyright (C) 1994, by WATCOM International Inc.  All rights    %
   %     reserved.  No part of this software may be reproduced or        %
   %     used in any form or by any means - graphic, electronic or       %
   %     mechanical, including photocopying, recording, taping or        %
   %     information storage and retrieval systems - except with the     %
   %     written permission of WATCOM International Inc.                 %
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

/*************************************************************************
 *
 * WHotKey -- Wrapper for the Windows 95 Hot-Key control.
 *
 *************************************************************************/

#ifndef _WHOTKEY_HPP_INCLUDED
#define _WHOTKEY_HPP_INCLUDED

#ifndef _WNO_PRAGMA_PUSH
#pragma pack(push,8);
#pragma enum int;
#endif

#ifndef _WCONTROL_HPP_INCLUDED
#  include "wcontrol.hpp"
#endif

//
// HotKey styles
//

#define WHKSDefault             ((WStyle)0x50000000L) // WS_VISIBLE|WS_CHILD

enum {
    WHKCombNone         = 0x0001,
    WHKCombS            = 0x0002,               // Shift
    WHKCombC            = 0x0004,               // Control
    WHKCombA            = 0x0008,               // Alt
    WHKCombSC           = 0x0010,               // Shift + Control
    WHKCombSA           = 0x0020,               // Shift + Alt
    WHKCombCA           = 0x0040,               // Control + Alt
    WHKCombSCA          = 0x0080,               // Shift + Control + Alt
};

class WCMCLASS WHotKey : public WControl {
    WDeclareSubclass( WHotKey, WControl );

    public:
    
        /**********************************************************
         * Constructors and destructors
         *********************************************************/

        WHotKey();
    
        ~WHotKey();

        /**********************************************************
         * Properties
         *********************************************************/

        // HotKey

        WHotKeyInfo GetHotKey();
        WBool SetHotKey( const WHotKeyInfo & hotKey );

        /**********************************************************
         * Methods
         *********************************************************/

        // SpecifyInvalidCombos

        void SpecifyInvalidCombos( WULong invalidCombos,
                                   WKeyState keyStateToUseInstead );

        /**********************************************************
         * Overrides
         *********************************************************/

        virtual WColor GetBackColor( WBool getResultingColor=TRUE ) const;

        virtual const WChar * InitializeClass();

        virtual WStyle GetDefaultStyle() const;

        virtual WBool CloneWindow( WStyle newStyle, WStyle newExStyle,
                                   void * data=NULL );
};

#ifndef _WNO_PRAGMA_PUSH
#pragma enum pop;
#pragma pack(pop);
#endif

#endif // _WHOTKEY_HPP_INCLUDED
