/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %     Copyright (C) 1996, by WATCOM International Inc.  All rights    %
   %     reserved.  No part of this software may be reproduced or        %
   %     used in any form or by any means - graphic, electronic or       %
   %     mechanical, including photocopying, recording, taping or        %
   %     information storage and retrieval systems - except with the     %
   %     written permission of WATCOM International Inc.                 %
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

/*************************************************************************
 *
 * WInputContext -- IME Input context
 *
 *************************************************************************/

#ifndef _WINPUTCX_HPP_INCLUDED
#define _WINPUTCX_HPP_INCLUDED

#ifndef _WNO_PRAGMA_PUSH
#pragma pack(push,8);
#pragma enum int;
#endif


#include "wdef.hpp"

#ifndef _WOBJECT_HPP_INCLUDED
#  include "wobject.hpp"
#endif

#define WIME_MODE_FULLSHAPE     0x0008 // IME_CMODE_FULLSHAPE
#define WIME_MODE_NATIVE        0x0001 // IME_CMODE_NATIVE
#define WIME_MODE_KATAKANA      0x0002 // IME_CMODE_KATAKANA
#define WIME_MODE_ALPHANUMERIC  0x0000 // IME_CMODE_ALPHANUMERIC
#define WIME_NO_CONVERSION      0x0100 // IME_CMODE_NOCONVERSION
#define WIME_MODE_ROMAN         0x0010 // IME_CMODE_ROMAN
#define WIME_MODE_SOFTKBD       0x0080 // IME_CMODE_SOFTKBD
#define WIME_MODE_CHARCODE      0x0020 // IME_CMODE_CHARCHODE
#define WIME_MODE_HANJA         0x0040 // IME_CMODE_HANJACONVERT
#define WIME_MODE_EUDC          0x0200 // IME_CMODE_EUDC
#define WIME_MODE_SYMBOL        0x0400 // IME_CMODE_SYMBOL

// non-roman defines
#define WIME_DB_HIRAGANA (WIME_MODE_FULLSHAPE | WIME_MODE_NATIVE)
#define WIME_DB_KATAKANA (WIME_MODE_FULLSHAPE | WIME_MODE_NATIVE | WIME_MODE_KATAKANA)
#define WIME_DB_ALPHANUM (WIME_MODE_FULLSHAPE)
#define WIME_SB_KATAKANA (WIME_MODE_KATAKANA | WIME_MODE_NATIVE)

// The names are:
//      No Conversion
//      Double Byte Hiragana
//      Double Byte Katakana
//      Single Byte Katakana
//      Double Byte Alphanumeric
//      Single Byte Alphanumeric

#define WIME_NONE              (WIME_NO_CONVERSION)
// roman defines
#define WIME_DB_HIRAGANA_ROMAN (WIME_MODE_FULLSHAPE | WIME_MODE_NATIVE | WIME_MODE_ROMAN)
#define WIME_DB_KATAKANA_ROMAN (WIME_MODE_KATAKANA | WIME_MODE_FULLSHAPE | WIME_MODE_NATIVE | WIME_MODE_ROMAN)
#define WIME_SB_KATAKANA_ROMAN (WIME_MODE_KATAKANA | WIME_MODE_NATIVE | WIME_MODE_ROMAN)
#define WIME_DB_ALPHANUM_ROMAN (WIME_MODE_FULLSHAPE | WIME_MODE_ROMAN)

#define WIME_SB_ALPHANUM       (WIME_MODE_ALPHANUMERIC)

class WCMCLASS WInputContext : public WObject {
    WDefineSubclass( WInputContext, WObject );

    public:

        /**************************************************************
         * Constructors and destructors
         **************************************************************/
    
        WInputContext();
        WInputContext( const WInputContext &ctx );
        ~WInputContext();

        WInputContext & operator=( const WInputContext &ctx );

        /**************************************************************
         * Methods
         **************************************************************/

         WBool Create();

         WBool Destroy();

        /**************************************************************
         * Properties
         **************************************************************/

        // InputMode
        //
        //      The input mode for this input context.

        WDWord GetInputMode() const;
        WBool      SetInputMode( WDWord mode );

        WInputContextHandle     GetHandle() const;

        /**************************************************************
         * Data members
         **************************************************************/

    private:
        WInputContextHandle     _inputContext;

};


#ifndef _WNO_PRAGMA_PUSH
#pragma enum pop;
#pragma pack(pop);
#endif

#endif // _WINPUTCX_HPP_INCLUDED
