/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %     Copyright (C) 1994, by WATCOM International Inc.  All rights    %
   %     reserved.  No part of this software may be reproduced or        %
   %     used in any form or by any means - graphic, electronic or       %
   %     mechanical, including photocopying, recording, taping or        %
   %     information storage and retrieval systems - except with the     %
   %     written permission of WATCOM International Inc.                 %
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

/*************************************************************************
 *
 * WMDIChild -- Wrapper for the Windows 95 MDI child control.
 *
 *   Events:
 *
 *************************************************************************/

#ifndef _WMDICHLD_HPP_INCLUDED
#define _WMDICHLD_HPP_INCLUDED

#ifndef _WNO_PRAGMA_PUSH
#pragma pack(push,8);
#pragma enum int;
#endif

#ifndef _WFORM_HPP_INCLUDED
#  include "wform.hpp"
#endif

class WMDIParent;

class WCMCLASS WMDIChild : public WForm {
    WDeclareSubclass( WMDIChild, WForm );
    
    public:

        /**************************************************************
         * Constructors and Destructors
         **************************************************************/

        WMDIChild();
    
        WMDIChild( WMDIParent * parent, WWindowHandle hdl=NULLHWND );
    
        virtual ~WMDIChild();

        /**************************************************************
         * Properties
         **************************************************************/

        // Active
        //
        //     Overrides the WForm behaviour and returns TRUE if the
        //     window is the active MDI child.

        virtual WBool GetActive() const;

        /**************************************************************
         * Methods
         **************************************************************/

        // Activate
        //
        //     Overrides the WForm behaviour and activates the child.

        virtual WBool Activate() const;

        // Maximize

        WBool Maximize();

        // Restore

        WBool Restore();

        /**************************************************************
         * Message processing
         **************************************************************/
        
        virtual WBool ProcessMessage( const WMessage & msg,
                                      WLong & returns );
    
        virtual WBool DefaultProcess( const WMessage & msg,
                                      WLong & returns );
    
        /**************************************************************
         * Others
         **************************************************************/
        
        virtual int operator ==( const WMDIChild & obj ) const;
        virtual int operator !=( const WMDIChild & obj ) const;

        WBool EraseBackgroundHandler( WWindow * window,
                                      WPaintEventData * event );

        WBool PaintHandler( WWindow * window, WPaintEventData * event );

        WBool CreateHandler( WWindow * window, WEventData * event );

        /**************************************************************
         * Overrides
         **************************************************************/
        
        virtual WBool MakeWindow( WWindow * parent, WUInt id,
                                  const WChar * className,
                                  const WChar * title, const WRect & r,
                                  WStyle wstyle, WStyle exStyle,
                                  void * data=NULL );

        virtual WBool FilterMessage( WMessage & msg, WBool & eatMessage );

        // Destroy (overridden)

        virtual WBool Destroy();

        /**************************************************************
         * Data members
         **************************************************************/
        
    private:
        WMDIParent *            _MDIParent;
        WBool                   _createVisible;
};

#ifndef _WNO_PRAGMA_PUSH
#pragma enum pop;
#pragma pack(pop);
#endif

#endif // _WMDICHLD_HPP_INCLUDED
