#ifndef _DTJCONT_HPP
#define _DTJCONT_HPP

#include "dtjcomp.hpp"
class DTJLayoutManager;

class METAEXPORTCLASSDEF DTJContainer : public DTJComponent
{
    public:
        DTJContainer( const MetaObject * pMetaObj );
        virtual ~DTJContainer();

	virtual WJavaObject* GetWJavaObject() const;

	void SetForm( DTFormEditBase * form );
        
	virtual WBool		Create( WWindow * parent, const WRect & r,
				        const WChar * text, WStyle style,
					WStyle exStyle, void * data = NULL,
					WBool isLoading = FALSE );

	virtual void GenFileFragment( MMFileFragment fg, WStringList &,
				      MMFileGenerationParms * parms ) const;

	//
	// DefaultStyle
	//
	//    The style with which new instances are created.  Default
	//    implementation returns GetWindow()->GetDefaultStyle().

	virtual WStyle		GetDefaultStyle() const;

	//
	// DefaultExtendedStyle
	//
	//    See DefaultStyle

	virtual WStyle		GetDefaultExtendedStyle() const;

	//
	// FDX
	//

	virtual WBool GetHasFDX() const;
	virtual WBool GetFDXGenerateClass() const;
	virtual void SetFDXGenerateClass( WBool );
	virtual WString GetFDXClassName() const;
	virtual void SetFDXClassName( const WString & name );

        virtual void GenerateCode( MMCodeGeneration mmCodeGen,
				   ostream& src,
				   MMCodeGenerationParms& pGenParms );

	virtual void AmendPopup( WPopupMenu *, WBool before );

	virtual void SetLayoutManager( DTJLayoutManager * layoutManager );

	virtual DTJLayoutManager * GetLayoutManager() const;

	virtual void PropertyChanged( const WString & name );

	virtual WString GetObjectLayoutName( DTJObject * object ) const;
        
        virtual void Layout();

	virtual WBool RemoveChild( DTObject * );

	virtual WBool AddChild( DTObject *, int after = -1 );
        
        virtual void SetNeedLayout( WBool needLayout );

	static PropertyInspectObject * MakePropEditor_FDX(
			const MMProperty * prop, PropertyInspector * prt,
			WUInt numObject, DTObject ** objects );

    protected:
	virtual void GenerateAllImports( ostream& src,
				         MMJCodeGenerationParms* pGenParms ) const;
	virtual void GenerateImports( ostream& src,
			  	      MMJCodeGenerationParms* pGenParms ) const;
	virtual void GenerateConstructor( ostream& src,
			  	          MMJCodeGenerationParms* pGenParms ) const;
    	virtual void GenerateCreate( const WString& objPrefix,
	                             ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;
    	virtual void GenerateCreate( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;
    	virtual void GenerateDestroy( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;
	virtual void GenerateEventHandler( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;
	virtual void GenerateAllListeners( ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;
	virtual void GenerateListener( const DTUserFunctionVect& eventList,
				    ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;
	virtual void GenerateMembersDeclarations( ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;
	virtual void GenerateDeclaration( ostream& src,
					  DTJObject* pDTJObject,
				    MMJCodeGenerationParms * pGenParms ) const;
	virtual void GenerateCheckboxGroupDeclarations( ostream& src ) const;
	virtual void BuildCheckboxGroupList( DTJObject* pDJTObject,
					     WStringList& groupList ) const;

        virtual void GenerateLayoutManager( ostream& src,
			            MMJCodeGenerationParms* pGenParms ) const;
        virtual void GenerateLayoutCode( ostream& src,
			            MMJCodeGenerationParms* pGenParms ) const;
        virtual void GenerateControlAddition( ostream& src,
			            MMJCodeGenerationParms* pGenParms ) const;
        virtual void GenerateControlCreation( ostream& src,
			            MMJCodeGenerationParms* pGenParms ) const;
        virtual void GenerateTransactionConnect( ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;
        virtual void GenerateTransactionCreate( ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;
        virtual void GenerateQueryCreate( ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;
        virtual void GenerateQueryAttach( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;
        virtual void GenerateSetSQL( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;
        virtual void GenerateAutoOpenQuery( ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;
        virtual void GenerateExtraProperties( ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;

        virtual void GenerateAutoProperties( ostream& src,
			            MMJCodeGenerationParms* pGenParms ) const;
	virtual void GenerateForm( ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateClassDef( ostream& src,
		   		    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateUserFunctions( ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateAllEvents( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateFDXMethods( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateAppChildSerialized( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateAppChildHTML( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateAppChildMethods( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateCustomImportBlock( DTObject* pDTObj,
				    ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateChildMethods( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateChildClasses( DTJObject* pDTJObject,
				    ostream& src,
				    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateParentMemberInitialize( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;

	virtual void GenerateMenus( ostream& src,
			  	    MMJCodeGenerationParms* pGenParms ) const;

	WBool HandleTabEditMenuClick( WMenuItem *, WEventData * );

    private:
	void FillEventList( MMJCodeGenerationParms * pGenParms );

    private:
	DTUserFunctionVect 	_eventList;
	WStringList *		_listenerTypeList;
	WMenuItem *		_tabEditMenuItem;
	DTJLayoutManager *	_layoutManager;
	WBool			_needLayout;
};

#define NUM_AWT_10_EVENTS 29

typedef struct {
    const WChar * identifier;
    const WChar * id;
} DefaultAWT10EventMapStruct;

extern DefaultAWT10EventMapStruct DefaultAWT10EventMap[NUM_AWT_10_EVENTS];

// needed for mdreader
typedef DTJContainer DTjava__dot__awt__dot__Container__dot__102;

#endif // _DTJCONT_HPP
