#ifndef _DTJTYPED_HPP
#define _DTJTYPED_HPP

#include <stddef.h>
#include "dttyped.hpp"

#define TYPE_ALIAS( _base, _alias )	\
typedef DT ## _base DT ## _alias;

class METAEXPORTCLASSDEF DTType_JRect : public DTType_WRect {
    public:
    	DTType_JRect( const MMType * imp ) : DTType_WRect( imp ) {}
	~DTType_JRect() {}

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );
};

class METAEXPORTCLASSDEF DTType_Color : public DTType_WColor {
    public:
	DTType_Color( const MMType * imp );
	~DTType_Color();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	virtual WBool		    operator==( const DTType & o );

	virtual WBool		    Set( const WColor & val );
	virtual const WColor &	    Get() const;

	virtual WBool		    Browse( WWindow * parent, DTObject * obj );

	virtual WUInt		    GetNumValues();
	virtual WString		    GetValue( WUInt i );

	static DTType *		    Instantiate( const MMType * );
};


TYPE_ALIAS( Type_Color, Type_const_Color__ref_ );

class METAEXPORTCLASSDEF DTType_JFont : public DTType {
    public:
	DTType_JFont( const MMType * imp );
	~DTType_JFont();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	virtual WBool		    operator==( const DTType & o );

	virtual WBool		    Set( const WFont & val );
	virtual const WFont &	    Get() const;

	virtual WBool		    Browse( WWindow * parent, DTObject * obj );

	static DTType *		    Instantiate( const MMType * );
    private:
	WFont		_value;
};

TYPE_ALIAS( Type_JFont, Type_const_JFont__ref_ );

class METAEXPORTCLASSDEF DTType_WString_Or_Null : public DTType_WString {
    public:
    	DTType_WString_Or_Null( const MMType * imp );
	~DTType_WString_Or_Null();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );

	void MarkAsNonNull() { _null = false; }

    protected:
    	WBool		_null;
};

class METAEXPORTCLASSDEF DTType_Color_Or_Null : public DTType_Color {
    public:
	DTType_Color_Or_Null( const MMType * imp );
	~DTType_Color_Or_Null();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );

	void MarkAsNonNull() { _null = false; }

    protected:
    	WBool		_null;
};

class METAEXPORTCLASSDEF DTType_JFont_Or_Null : public DTType_JFont {
    public:
	DTType_JFont_Or_Null( const MMType * imp );
	~DTType_JFont_Or_Null();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );

	void MarkAsNonNull() { _null = false; }

    protected:
    	WBool		_null;
};

class METAEXPORTCLASSDEF DTType_WChar_Or_Null : public DTType_WChar {
    public:
	DTType_WChar_Or_Null( const MMType * imp );
	~DTType_WChar_Or_Null();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );

	void MarkAsNonNull() { _null = false; }

    protected:
    	WBool		_null;
};

class METAEXPORTCLASSDEF DTType_JBool : public DTBoolType {
    public:
	DTType_JBool( const MMType * imp ) : DTBoolType( imp ) {}
	~DTType_JBool() {}

	static DTType *	    MakeDTJBool( const MMType * );
	static DTType *	    Instantiate( const MMType * mm ) { return MakeDTJBool( mm ); }

	virtual WUInt	    GetNumValues() { return 2; }
};

class METAEXPORTCLASSDEF DTType_JBool_Or_Null : public DTType_JBool {
    public:
	DTType_JBool_Or_Null( const MMType * imp );
	~DTType_JBool_Or_Null();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );

	void MarkAsNonNull() { _null = false; }

    protected:
    	WBool		_null;
};

class METAEXPORTCLASSDEF DTType_WInt_Or_Null : public DTType_WInt {
    public:
	DTType_WInt_Or_Null( const MMType * imp );
	~DTType_WInt_Or_Null();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );

	void MarkAsNonNull() { _null = false; }

    protected:
    	WBool		_null;
};

class METAEXPORTCLASSDEF DTEnumType_Or_Null: public DTEnumType {
    public:
	DTEnumType_Or_Null( const MMType * imp );
	~DTEnumType_Or_Null();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );

	void MarkAsNonNull() { _null = false; }

    protected:
    	WBool		_null;
};

class METAEXPORTCLASSDEF DTType_WShort_Or_Null : public DTType_WShort {
    public:
	DTType_WShort_Or_Null( const MMType * imp );
	~DTType_WShort_Or_Null();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );

	void MarkAsNonNull() { _null = false; }

    protected:
    	WBool		_null;
};

class METAEXPORTCLASSDEF DTType_WUShort_Or_Null : public DTType_WUShort {
    public:
	DTType_WUShort_Or_Null( const MMType * imp );
	~DTType_WUShort_Or_Null();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );

	void MarkAsNonNull() { _null = false; }

    protected:
    	WBool		_null;
};

class METAEXPORTCLASSDEF DTType_WLong_Or_Null : public DTType_WLong {
    public:
	DTType_WLong_Or_Null( const MMType * imp );
	~DTType_WLong_Or_Null();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );

	void MarkAsNonNull() { _null = false; }

    protected:
    	WBool		_null;
};

class METAEXPORTCLASSDEF DTType_WUInt_Or_Null : public DTType_WUInt {
    public:
	DTType_WUInt_Or_Null( const MMType * imp );
	~DTType_WUInt_Or_Null();

	virtual WBool		    Textify( WString &, DT_Textify ) const;
	virtual WBool		    DeTextify( const WString &, DT_Textify );

	static DTType *		    Instantiate( const MMType * );

	void MarkAsNonNull() { _null = false; }

    protected:
    	WBool		_null;
};

extern WBool METAEXPORTDEF SearchAndReplace( WString & given,
			const WChar * search,
                        const WChar * replace );

extern void METAEXPORTDEF TranslateString( WString &outStr,
                                           const WString &inStr );

#endif // _DTJTYPED_HPP
