/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %     Copyright (C) 1994, by WATCOM International Inc.  All rights    %
   %     reserved.  No part of this software may be reproduced or        %
   %     used in any form or by any means - graphic, electronic or       %
   %     mechanical, including photocopying, recording, taping or        %
   %     information storage and retrieval systems - except with the     %
   %     written permission of WATCOM International Inc.                 %
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#ifndef _WOLETYPE_HPP_INCLUDED
#define _WOLETYPE_HPP_INCLUDED

#ifndef _WNO_PRAGMA_PUSH
#pragma pack(push,8);
#pragma enum int;
#endif

#define WOLE_IFACE(name)        struct _w##name { int dummy; }; \
                                typedef const struct _w##name * name

WOLE_IFACE(WPIUnknown );

WOLE_IFACE(WPIDispatch );

WOLE_IFACE(WPIStream );

WOLE_IFACE(WPIStorage );

WOLE_IFACE(WPIClassFactory );

WOLE_IFACE(WPIClassFactory2 );

WOLE_IFACE(WPIDataObject );

WOLE_IFACE(WPICursor );

WOLE_IFACE(WPIFont );

WOLE_IFACE(WPIPicture );

WOLE_IFACE(WPIErrorInfo );

// **** The following represent a minimal cover for the data types
// **** defined in the OLE Automation header files

typedef unsigned int WDispIndex;

typedef long WDispID;

typedef WDispID WMemberID;

typedef long WScode;

typedef long WHRESULT;

typedef short WVARIANT_BOOL;

typedef double WDate;

typedef long char * WBStr;

/**********************************************************
 * Typedefs for standard scalar types
 *********************************************************/
typedef unsigned long   WOLE_COLOR;
typedef long            WOLE_XPOS_PIXELS;
typedef long            WOLE_YPOS_PIXELS;
typedef long            WOLE_XSIZE_PIXELS;
typedef long            WOLE_YSIZE_PIXELS;
typedef long            WOLE_XPOS_HIMETRIC;
typedef long            WOLE_YPOS_HIMETRIC;
typedef long            WOLE_XSIZE_HIMETRIC;
typedef long            WOLE_YSIZE_HIMETRIC;
typedef float           WOLE_XPOS_CONTAINER;
typedef float           WOLE_YPOS_CONTAINER;
typedef float           WOLE_XSIZE_CONTAINER;
typedef float           WOLE_YSIZE_CONTAINER;
typedef WBool           WOLE_OPTEXCLUSIVE;
typedef WBool           WOLE_CANCELBOOL;
typedef WBool           WOLE_ENABLEDEFAULTBOOL;
typedef unsigned int    WOLE_HANDLE;

enum WOLE_TRISTATE { WtriUnchecked = 0, WtriChecked = 1, WtriGray = 2 };

typedef struct {
    unsigned long       dwLowDateTime;
    unsigned long       dwHighDateTime;
} WFileTime;

typedef struct {
    unsigned long       Lo;
    long                Hi;
} WCurrency;  // CY

typedef struct {
    unsigned long       cElements;
    long                lLbound;
} WSafeArrayBound;

typedef struct {
    unsigned short      cDims;
    unsigned short      fFeatures;
    unsigned long       cbElements;
    unsigned long       cLocks;
    void                *pvData;
    WSafeArrayBound     rgsabound[1];
} WSafeArray;

// the following are the feature flags for the WSafeArray
#define WFADF_AUTO      ( 0x1 )
#define WFADF_STATIC    ( 0x2 )
#define WFADF_EMBEDDED  ( 0x4 )
#define WFADF_FIXEDSIZE ( 0x10 )
#define WFADF_BSTR      ( 0x100 )
#define WFADF_UNKNOWN   ( 0x200 )
#define WFADF_DISPATCH  ( 0x400 )
#define WFADF_VARIANT   ( 0x800 )
#define WFADF_RESERVED  ( 0xf0e8 )

typedef struct {
    WDWord      LowPart;
    WLong       HighPart;
} WLARGE_INTEGER;

typedef struct {
    WDWord      LowPart;
    WDWord      HighPart;
} WULARGE_INTEGER;

#define WLISet32(li, v) ((li).HighPart = (v) < 0 ? -1 : 0, (li).LowPart = (v))
#define WULISet32(li, v) ((li).HighPart = 0, (li).LowPart = (v))

#define WLongFromLI(l, li) ( (l) = (li).LowPart)
#define WULongFromULI(ul, li) ((ul) = (li).LowPart)

typedef struct {
    WDWord      Data1;
    WUShort     Data2;
    WUShort     Data3;
    WByte       Data4[ 8 ];
} WGUID;

typedef WGUID WIID;
typedef WGUID WCLSID;

extern "C" const WGUID WGUID_NULL;
#define WCLSID_NULL WGUID_NULL
#define WIID_NULL WGUID_NULL

WBool WEXPORT WIsEqualWGUID( const WGUID & guid1, const WGUID & guid2 );
#define WIsEqualWCLSID WIsEqualWGUID
#define WIsEqualWIID WIsEqualWGUID

extern WBool WEXPORT operator==(const WGUID& guid1, const WGUID& guid2 );
extern WBool WEXPORT operator!=(const WGUID& guid1, const WGUID& guid2 );

typedef struct {
    WDWord      tdSize;
    WUShort     tdDriverNameOffset;
    WUShort     tdDeviceNameOffset;
    WUShort     tdPortNameOffset;
    WUShort     tdExtDevmodeOffset;
    WByte       tdData[ 1 ];
} WDVTargeTDevice;

#ifndef _WNO_PRAGMA_PUSH
#pragma enum pop;
#pragma pack(pop);
#endif

#endif // _WOLETYPE_HPP_INCLUDED
